#!/usr/bin/env python
from __future__ import print_function

def osfi_msg(type, txt):
    """Write OSFI-formatted message"""
    import sys
    print("{:<10}| {} | 3.3".format(type, txt))
    sys.stdout.flush() # For Python 2.7 with __future__

def findSingleParam(elem, path, file):
    comps = path.split(".")
    comps[-1] = "parameter[@name='{}']".format(comps[-1])
    expr = "/".join(comps)
    #print(repr(expr))
    param = elem.findall(expr)
    if len(param) != 1:
        raise IndexError("Expected to find exactly one param with path {0} in file {1}, found {2}"
                         .format(path, file, len(param)))
    return param[0]

def GeometryModule(args):
    import xml.etree.ElementTree as ET

    if len(args) != 3:
        raise IndexError("Exactly 3 arguments are required per E2E-ICD")
    [cfg, inp, out] = [x.split(",") for x in args]
    if len(cfg) != 2:
        raise IndexError("Exactly 2 config files are required per E2E-ICD")
    osfi_msg("Info", "GCF: " + cfg[0])
    osfi_msg("Info", "LCF: " + cfg[1])
    osfi_msg("Info", "Inputs: " + repr(inp))
    osfi_msg("Info", "Outputs: " + repr(out))

    [gcf, lcf] = map(ET.parse, cfg)
    # Read something from each config file
    globalParam = int(findSingleParam(gcf.getroot(), 'orbit.iniRaan', cfg[0]).text)
    osfi_msg("Info", "Choice of case for orbit data files: #{}".format(globalParam) )

    localParam = int(findSingleParam(lcf.getroot(), 'Geometry.nominal_floris_pix_size', cfg[1]).text)
    osfi_msg("Info", "Nominal pixel size on ground: {}".format(localParam))

    # Read input files: orbit data folder and S/C inertia model file
    if len(inp) != 2:
        raise IndexError("Exactly 2 inputs are required: orbit data files folder and S/C inertia model file")
    orbFile = "{}/ORB_CASES_RAAN0_{}.OEM".format(inp[0], globalParam)
    osfi_msg("Info", "Reading orbit data from {}...".format(orbFile))
    try:
        with open(orbFile, "rt") as orbData:
            pass
    except IOError as e:
        raise ValueError("Could not open orbit data file")
    osfi_msg("Info", "Reading S/C inertia model from {}...".format(inp[1]))
    try:
        with open(inp[1], "rt") as scInertiaData:
            pass
    except IOError as e:
        raise ValueError("Could not open S/C inertia data file")

    if len(out) != 1:
        raise IndexError("Geometry module generates exactly one input")
    osfi_msg("Info", "Computing geometry and writing into {}...".format(out[0]))
    with open(out[0], "wt") as f:
        import time
        print("Geometry data with RAAN0={0} deg and pixel size {1}".format(globalParam, localParam), file=f)
        for i in range(1,localParam+1):
            time.sleep(0.01)
            print("Data line {} of {}".format(i,localParam), file=f)
            osfi_msg("Progress", "{} of {}".format(i,localParam))

    osfi_msg("Info", "Completed successfully")

if __name__ == "__main__":
    import sys
    if len(sys.argv) == 1: # For testing outside openSF
        sys.argv = ["Geometry.py","globalConfiguration.xml,GEO.CI.02_check.xml","orbit_data,sc_inertia.nc","geometry.dat"]
    try:
        GeometryModule(sys.argv[1:])
    except SystemExit as e:
        raise
    except BaseException as e:
        osfi_msg("Error", str(e))
        sys.exit(1)
