#!/usr/bin/env python
from __future__ import print_function

def osfi_msg(type, txt):
    """Write OSFI-formatted message"""
    import sys
    print("{:<10}| {} | 3.3".format(type, txt))
    sys.stdout.flush() # For Python 2.7 with __future__

def findSingleParam(elem, path, file):
    comps = path.split(".")
    comps[-1] = "parameter[@name='{}']".format(comps[-1])
    expr = "/".join(comps)
    #print(repr(expr))
    param = elem.findall(expr)
    if len(param) != 1:
        raise IndexError("Expected to find exactly one param with path {0} in file {1}, found {2}"
                         .format(path, file, len(param)))
    return param[0]

def ProductGenModule(args):
    import xml.etree.ElementTree as ET
    import os
    import os.path

    if len(args) != 3:
        raise IndexError("Exactly 3 arguments are required per E2E-ICD")
    [cfg, inp, out] = [x.split(",") for x in args]
    if len(cfg) != 2:
        raise IndexError("Exactly 2 config files are required per E2E-ICD")
    osfi_msg("Info", "GCF: " + cfg[0])
    osfi_msg("Info", "LCF: " + cfg[1])
    osfi_msg("Info", "Inputs: " + repr(inp))
    osfi_msg("Info", "Outputs: " + repr(out))

    [gcf, lcf] = map(ET.parse, cfg)
    # Read something from each config file
    globalParam = int(findSingleParam(gcf.getroot(), 'orbit.iniRaan', cfg[0]).text)
    osfi_msg("Info", "Choice of case for orbit data files: #{}".format(globalParam) )

    localParam = int(findSingleParam(lcf.getroot(), 'n_sessions', cfg[1]).text)
    osfi_msg("Info", "Session # to use (0=last): {}".format(localParam))

    # Read input files: Geometry output
    if len(inp) != 3:
        raise IndexError("Exactly 3 inputs are required: case geometry, scenes folder and IP FW")
    osfi_msg("Info", "Reading geometry data from {}...".format(inp[0]))
    try:
        with open(inp[0], "rt") as geomData:
            pass
    except IOError as e:
        raise ValueError("Could not open geometry data file")
    
    iScen = localParam
    if iScen == 0:
        osfi_msg("Warning", "Last session selected, result may not be reproducible")
        scenFiles = [ x for x in os.listdir(inp[1]) if x.startswith("scen_") and x.endswith(".dat") ]
        osfi_msg("Debug", "Session files found: " + repr(scenFiles))
        iScen = max(int(x[5:-4]) for x in scenFiles)
        osfi_msg("Debug", "Session # selected: {}".format(iScen))
    sceneFile = os.path.join(inp[1],"scen_{}.dat".format(iScen))
    osfi_msg("Info", "Reading scene data from {}...".format(sceneFile))
    try:
        with open(sceneFile, "rt") as scenData:
            pass
    except IOError as e:
        raise ValueError("Could not open scene data data file")
    
    if os.path.isfile(inp[2]):
        osfi_msg("Info", "Loading IP firmware blob from {}...".format(inp[2]))
    else:
        osfi_msg("Warning", "IP firmware blob at {} does not exist, using internal model".format(inp[2]))

    if len(out) != 2:
        raise IndexError("Product module generates exactly two output")
    #osfi_msg("Info", "Computing scene and writing into {} (summary) and {} (data)...".format(out[0],out[1]))
    with open(out[0], "wt") as fcov, open(out[1], "wt") as frt:
        import time
        print("Coverage results with RAAN0={} deg and scene {}".format(globalParam, localParam), file=fcov)
        print("Revisit time results with RAAN0={} deg and scene {}".format(globalParam, localParam), file=frt)

    osfi_msg("Info", "Completed successfully")

if __name__ == "__main__":
    import sys
    if len(sys.argv) == 1: # For testing outside openSF
        sys.argv = ["ProductGenModule.py","globalConfiguration.xml,PEM.CI.02_PEMConfig.xml",
                    "geometry.dat,sg_data_outs,ip_fw.bin",
                    "coverage.dat,revisit.dat"]
    try:
        ProductGenModule(sys.argv[1:])
    except SystemExit as e:
        raise
    except BaseException as e:
        osfi_msg("Error", str(e))
        sys.exit(1)
