#!/usr/bin/env python
from __future__ import print_function

def osfi_msg(type, txt):
    """Write OSFI-formatted message"""
    import sys
    print("{:<10}| {} | 3.3".format(type, txt))
    sys.stdout.flush() # For Python 2.7 with __future__

def findSingleParam(elem, path, file):
    comps = path.split(".")
    comps[-1] = "parameter[@name='{}']".format(comps[-1])
    expr = "/".join(comps)
    #print(repr(expr))
    param = elem.findall(expr)
    if len(param) != 1:
        raise IndexError("Expected to find exactly one param with path {0} in file {1}, found {2}"
                         .format(path, file, len(param)))
    return param[0]

def SceneGenModule(args):
    import xml.etree.ElementTree as ET

    if len(args) != 3:
        raise IndexError("Exactly 3 arguments are required per E2E-ICD")
    [cfg, inp, out] = [x.split(",") for x in args]
    if len(cfg) != 2:
        raise IndexError("Exactly 2 config files are required per E2E-ICD")
    osfi_msg("Info", "GCF: " + cfg[0])
    osfi_msg("Info", "LCF: " + cfg[1])
    osfi_msg("Info", "Inputs: " + repr(inp))
    osfi_msg("Info", "Outputs: " + repr(out))

    [gcf, lcf] = map(ET.parse, cfg)
    # Read something from each config file
    globalParam = int(findSingleParam(gcf.getroot(), 'orbit.iniRaan', cfg[0]).text)
    osfi_msg("Info", "Choice of case for orbit data files: #{}".format(globalParam) )

    localParam = int(findSingleParam(lcf.getroot(), 'LOCAL.num_parallel_cores', cfg[1]).text)
    osfi_msg("Info", "Number of cores to use: {}".format(localParam))

    osfi_msg("Warning", "No could cover data available, using default model")

    # Read input files: Geometry output
    if len(inp) != 1:
        raise IndexError("Exactly 1 input is required: geometry output")
    osfi_msg("Info", "Reading geometry data from {}...".format(inp[0]))
    try:
        with open(inp[0], "rt") as geomData:
            pass
    except IOError as e:
        raise ValueError("Could not open geometry data file")

    if len(out) != 2:
        raise IndexError("SG module generates exactly two outputs")
    osfi_msg("Info", "Computing scene and writing into {} (summary) and {} (data)...".format(out[0],out[1]))
    with open(out[0], "wt") as fsum:
        import time
        import os
        import os.path
        import shutil
        print("Scene data with RAAN0={0} deg, created using {1} cores".format(globalParam, localParam), file=fsum)
        if os.path.isdir(out[1]):
            shutil.rmtree(out[1])
        os.mkdir(out[1])
        nscen = 5
        for i in range(1, nscen+1):
            time.sleep(1.0/localParam)
            print("Summary for scene {} of {}".format(i, nscen), file=fsum)
            dataFileName = os.path.join(out[1], "scen_{}.dat".format(i))
            osfi_msg("Info", "Writing data for scene {} into {}".format(i, dataFileName))
            with open(dataFileName, "wt") as fdet:
                print("Data for scene {} of {}".format(i, nscen), file=fdet)
            osfi_msg("Progress", "{} of {}".format(i, nscen))    

    osfi_msg("Info", "Completed successfully")

if __name__ == "__main__":
    import sys
    if len(sys.argv) == 1: # For testing outside openSF
        sys.argv = ["SceneGenModule.py","globalConfiguration.xml,SGM.CI.02.xml","geometry.dat","sg_data_summary.txt,sg_data_outs"]
    try:
        SceneGenModule(sys.argv[1:])
    except SystemExit as e:
        raise
    except BaseException as e:
        osfi_msg("Error", str(e))
        sys.exit(1)
